package sialocacao.disciplina.database;


/**
 * <p>Title: DisciplinaDatabase.java </p>
 * <p>Description: Implementa a camada de persistncia de Disciplina. </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III </p>
 * @author: Carlos Andreazza Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 1.0
 */

import java.sql.*;
import java.util.*;

import sialocacao.*;
import sialocacao.disciplina.*;
import sialocacao.disciplina.excecoes.*;
import sialocacao.sala.Categoria;

public class DisciplinaDatabase implements IDisciplinaDatabase {

  private Database poolBD;


  /*
   * Construtor para carregar driver jdbc.
   */
  public DisciplinaDatabase() throws Exception {
    poolBD = Database.instance();
  }

  /*
   * Insere disciplina no sistema.
   * @param descricao
   * @param cargaHoraria
   * @param ementa
   */
  public void inserirDisciplina(String descricao, int cargaHoraria, String ementa, Map preferencias)
      throws SIAlocacaoException {

    Connection con = null;

    try {
      con = poolBD.getConnection();
      PreparedStatement cmd = con.prepareStatement("INSERT INTO "+
      "disciplina (descricao, cargaHoraria, ementa) VALUES (?, ?, ?)");

      cmd.setString(1, descricao);
      cmd.setInt(2, cargaHoraria);
      cmd.setString(3, ementa);
      cmd.executeUpdate();

      PreparedStatement cmd3 = con.prepareStatement("SELECT CODIGO "+
      "FROM disciplina ORDER BY codigo DEsC");
      ResultSet rs = cmd3.executeQuery();
      int codDisciplina = 0;
      if(rs.next()) {
        codDisciplina = rs.getInt("codigo");
      }


      Iterator categorias = preferencias.keySet().iterator();
      while(categorias.hasNext()) {
        String codCategoria = (String)categorias.next();
        int preferencia = Integer.parseInt((String)preferencias.get(codCategoria));
        PreparedStatement cmd2 = con.prepareStatement("INSERT INTO "+
        "disciplina_categoria (codigoDisciplina, codigoCategoria , prioridade) VALUES (?, ?, ?)");

        cmd2.setInt(1, codDisciplina);
        cmd2.setInt(2, Integer.parseInt(codCategoria));
        cmd2.setInt(3, preferencia);
        cmd2.executeUpdate();
      }

      rs.close();
      cmd.close();

    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        poolBD.returnConnection(con);
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Altera disciplina no sistema.
   * @param disciplina
   */
  public void alterarDisciplina(Disciplina disciplina, Map prioridades)
      throws SIAlocacaoException, DisciplinaInexistenteException {

    Connection con = null;

    try {
      con = poolBD.getConnection();
      PreparedStatement cmd = con.prepareStatement( "UPDATE disciplina SET descricao=?, cargaHoraria=?, ementa=? WHERE codigo=?");

      cmd.setString(1, disciplina.getDescricao());
      cmd.setInt(2, disciplina.getCargaHoraria());
      cmd.setString(3, disciplina.getEmenta());
      cmd.setInt(4, disciplina.getCodigo());

      int num = cmd.executeUpdate();
      if (num == 0) {
        throw new DisciplinaInexistenteException(disciplina.getCodigo());
      }

      cmd.close();

      // remove as prioridades
      PreparedStatement cmd3 = con.prepareStatement("DELETE FROM disciplina_categoria "+
         "where codigoDisciplina = ?");
      cmd3.setInt(1, disciplina.getCodigo());
      cmd3.executeUpdate();
      cmd3.close();

      // insere as prioridades atualizadas
      Iterator categorias = prioridades.keySet().iterator();
      while(categorias.hasNext()) {
        String codCategoria = (String)categorias.next();
        int preferencia = Integer.parseInt((String)prioridades.get(codCategoria));
        PreparedStatement cmd2 = con.prepareStatement("INSERT INTO "+
        "disciplina_categoria (codigoDisciplina, codigoCategoria , prioridade) VALUES (?, ?, ?)");

        cmd2.setInt(1, disciplina.getCodigo());
        cmd2.setInt(2, Integer.parseInt(codCategoria));
        cmd2.setInt(3, preferencia);
        cmd2.executeUpdate();
        cmd2.close();
      }

    } catch (DisciplinaInexistenteException e) {
      throw e;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        poolBD.returnConnection(con);
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Exclui disciplina do sistema pelo cdigo.
   * @param codigo
   */
  public void excluirDisciplina(int codigo)
      throws SIAlocacaoException, DisciplinaInexistenteException {

    Connection con = null;

    try {
      con = poolBD.getConnection();

      // remove as prioridades
      PreparedStatement cmd2 = con.prepareStatement("DELETE FROM disciplina_categoria "+
         "where codigoDisciplina = ?");
      cmd2.setInt(1, codigo);
      cmd2.executeUpdate();

      PreparedStatement cmd = con.prepareStatement("DELETE FROM disciplina WHERE codigo=?");

      cmd.setInt(1, codigo);

      int num = cmd.executeUpdate();
      if (num == 0) {
        throw new DisciplinaInexistenteException(codigo);
      }

      cmd2.close();
      cmd.close();
    }catch (DisciplinaInexistenteException e) {
      throw e;
    }catch (SQLException e) {
      if(e.getMessage().indexOf("referential integrity") >= 0)
        throw new DisciplinaAlocadaException();
    }
    catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        poolBD.returnConnection(con);
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Busca disciplina no sistema pelo cdigo.
   * @param codigo
   * @return Disciplina
   */
  public Disciplina buscarDisciplina(int codigo)
      throws SIAlocacaoException, DisciplinaInexistenteException {

    Connection con  = null;

    try {
      con = poolBD.getConnection();
      PreparedStatement cmd = con.prepareStatement("SELECT descricao, cargaHoraria, ementa FROM disciplina WHERE codigo=?");

      cmd.setInt(1, codigo);

      ResultSet result = cmd.executeQuery();
      Disciplina disciplina;

      if (result.next()) {
        disciplina = new Disciplina (codigo,
                        result.getString("descricao"),
                        result.getInt("cargaHoraria"),
                        result.getString("ementa") );
      } else {
        throw new DisciplinaInexistenteException(codigo);
      }

      result.close();
      cmd.close();

      return disciplina;
    } catch (DisciplinaInexistenteException e) {
      throw e;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        poolBD.returnConnection(con);
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Busca disciplina no sistema pela descrio.
   * @param descricao
   * @return Disciplina
   */
  public Disciplina buscarDisciplina(String descricao)
      throws SIAlocacaoException, DisciplinaInexistenteException {

    Connection con = null;

    try {
      con = poolBD.getConnection();
      PreparedStatement cmd = con.prepareStatement("SELECT codigo, "+
        "cargaHoraria, ementa FROM disciplina WHERE descricao=?");

      cmd.setString(1, descricao);

      ResultSet result = cmd.executeQuery();
      Disciplina disciplina;

      if (result.next()) {
        disciplina = new Disciplina (result.getInt("codigo"),
                        descricao,
                        result.getInt("cargaHoraria"),
                        result.getString("ementa"));
      } else {
        throw new DisciplinaInexistenteException(descricao);
      }

      result.close();
      cmd.close();
      return disciplina;
    } catch (DisciplinaInexistenteException e) {
      throw e;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        poolBD.returnConnection(con);
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
}

  /*
   * Lista disciplinas do sistema.
   * @return ArrayList
   */
  public ArrayList listarDisciplinas() throws
       SIAlocacaoException, DisciplinaInexistenteException {

     Connection con = null;

     try {
       con = poolBD.getConnection();
       PreparedStatement cmd = con.prepareStatement("SELECT * FROM disciplina");
       ResultSet result = cmd.executeQuery();
       ArrayList disciplinas = new ArrayList();

       while (result.next()) {
         disciplinas.add(new Disciplina ( result.getInt("codigo"),
             result.getString("descricao"),
             result.getInt("cargaHoraria"),
             result.getString("ementa")));
       }

       result.close();
       cmd.close();
       return disciplinas;
     } catch (Exception e) {
       throw new SIAlocacaoException(e.getMessage(), e);
     } finally {
        try {
          poolBD.returnConnection(con);
        } catch (Exception e) {
          throw new SIAlocacaoException(e.getMessage(), e);
        }
     }
   }

   public Map buscarPrioridades(Disciplina disciplina, List categorias) throws
       SIAlocacaoException {
     Map prioridades = new HashMap();
     Connection con  = null;
     try {
       con = poolBD.getConnection();


       PreparedStatement cmd = con.prepareStatement("SELECT * FROM "+
           "disciplina_categoria WHERE codigoCategoria=? and codigoDisciplina=?");

       for(int i=0;i<categorias.size(); i++) {
          cmd.setInt(1, ((Categoria)categorias.get(i)).getCodigo());
          cmd.setInt(2, disciplina.getCodigo());
          ResultSet result = cmd.executeQuery();
          boolean achou = false;
          while(result.next()) {
              achou = true;
              String codCategoria = result.getString("codigoCategoria");
              String prioridade = result.getString("prioridade");
              prioridades.put(codCategoria, prioridade);
          }
          if(!achou) {
              prioridades.put(((Categoria)categorias.get(i)).getCodigo()+"", "2");
          }

          result.close();
       }
        cmd.close();
        return prioridades;
     } catch (Exception e) {
       throw new SIAlocacaoException(e.getMessage(), e);
     } finally {
        try {
          poolBD.returnConnection(con);
        } catch (Exception e) {
          throw new SIAlocacaoException(e.getMessage(), e);
        }
     }
   }

  /*
   * Associa disciplina a professor no sistema.
   * @param disciplina
   * @param codProfessor
   */
  public void associaProfessorDisciplina(Disciplina disciplina, int codProfessor)
      throws SIAlocacaoException, DisciplinaInexistenteException {

    Connection con  = null;

    try {
      con = poolBD.getConnection();
      PreparedStatement cmd = con.prepareStatement("UPDATE disciplina SET codigoProfessor=? WHERE codigo=?");

      if(codProfessor == 0) {
         cmd.setNull(1, 0);
      } else {
         cmd.setInt(1, codProfessor);
      }
      cmd.setInt(2, disciplina.getCodigo());

      int num = cmd.executeUpdate();
      if (num == 0) {
        throw new DisciplinaInexistenteException(disciplina.getCodigo());
      }
      cmd.close();
    } catch (DisciplinaInexistenteException e) {
      throw e;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
        try {
          poolBD.returnConnection(con);
        } catch (Exception e) {
          throw new SIAlocacaoException(e.getMessage(), e);
        }
    }
  }

  /*
   * Remove associacao de disciplina a professor no sistema.
   * @param codProfessor
   */
  public void removeProfessorDisciplina(int codProfessor) throws
       SIAlocacaoException, DisciplinaInexistenteException {

     Connection con = null;

     try {
       con = poolBD.getConnection();
       PreparedStatement cmd = con.prepareStatement("SELECT * FROM disciplina where codigoProfessor=?");

       cmd.setInt(1, codProfessor);
       ResultSet result = cmd.executeQuery();
       ArrayList disciplinas = new ArrayList();

       while (result.next()) {
          PreparedStatement cmd2 = con.prepareStatement("UPDATE disciplina SET codigoProfessor=? WHERE codigo=?");

          cmd2.setNull(1, 0);
          cmd2.setInt(2, result.getInt("codigo"));

          int num = cmd2.executeUpdate();
          if (num == 0) {
            throw new DisciplinaInexistenteException(result.getInt("codigo"));
          }
       }
       result.close();
       cmd.close();
     } catch (DisciplinaInexistenteException e) {
       throw e;
     } catch (Exception e) {
       throw new SIAlocacaoException(e.getMessage(), e);
     } finally {
        try {
          poolBD.returnConnection(con);
        } catch (Exception e) {
          throw new SIAlocacaoException(e.getMessage(), e);
        }
     }
  }

  /*
   * Busca codigo do profesor associado a disciplina no sistema.
   * @param codigo
   * @return int
   */
  public int buscarProfessorDeDisciplina(int codigo)
      throws SIAlocacaoException, DisciplinaInexistenteException {

    Connection con = null;

    try {
      con = poolBD.getConnection();
      PreparedStatement cmd = con.prepareStatement("SELECT codigoProfessor FROM disciplina WHERE codigo=?");

      cmd.setInt(1, codigo);

      ResultSet result = cmd.executeQuery();
      int codigoProfessor;

      if (result.next()) {
        String str = result.getString("codigoProfessor");
        if(str == null) {
           codigoProfessor = 0;
        } else {
           codigoProfessor = Integer.parseInt(str);
        }
      } else {
        throw new DisciplinaInexistenteException(codigo);
      }

      result.close();
      cmd.close();

      return codigoProfessor;
    } catch (DisciplinaInexistenteException e) {
      throw e;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
        try {
          poolBD.returnConnection(con);
        } catch (Exception e) {
          throw new SIAlocacaoException(e.getMessage(), e);
        }
    }
  }

  /*
   * Verifica se todas as disciplinas estao associadas a um professor
   * @return true
   */
  public boolean todasDisciplinasAssociadas()
      throws SIAlocacaoException{

    Connection con = null;

    try {
      con = poolBD.getConnection();

      PreparedStatement cmd = con.prepareStatement("SELECT codigoProfessor "+
        "FROM disciplina");

      ResultSet result = cmd.executeQuery();

      while(result.next()){
        if(result.getString(1) == null){
          return false;
        }
      }

      result.close();
      cmd.close();

      return true;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
        try {
          poolBD.returnConnection(con);
        } catch (Exception e) {
          throw new SIAlocacaoException(e.getMessage(), e);
        }
    }
  }
}