package sialocacao.professor.database;

/**
 * <p>Title: ProfessorDatabase.java </p>
 * <p>Description: Implementa a camada de persistncia de professor. </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III </p>
 * @author: Carlos Andreazza Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 2.0
*/

import java.sql.*;
import java.util.*;

import sialocacao.*;
import sialocacao.professor.*;
import sialocacao.professor.excecoes.*;

public class ProfessorDatabase implements IProfessorDatabase {

  private Database poolBD;

  /*
   * Construtor para carregar driver jdbc.
   */
  public ProfessorDatabase() throws Exception {
    poolBD = Database.instance();
  }

  /*
   * Insere professor no sistema.
   * @param nome
   * @param cpf
   */
  public void inserirProfessor(String nome, String cpf) throws SIAlocacaoException {

    Connection con = null;
    try {
      con = poolBD.getConnection();
      PreparedStatement cmd =
          con.prepareStatement("INSERT INTO professor (nome," +
                                           "cpf) VALUES (?, ?)");

      cmd.setString(1, nome);
      cmd.setString(2, cpf);

      cmd.executeUpdate();

      cmd.close();

    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        poolBD.returnConnection(con);
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Altera professor no sistema.
   * @param professor
   */
  public void alterarProfessor(Professor professor) throws SIAlocacaoException,
      ProfessorInexistenteException {

    Connection con = null;
    try {
      con = poolBD.getConnection();
      PreparedStatement cmd =
          con.prepareStatement(
          "UPDATE professor SET nome=?, cpf=? " +
          "WHERE codigo=?");

      cmd.setString(1, professor.getNome());
      cmd.setString(2, professor.getCpf());
      cmd.setInt(3, professor.getCodigo());

      int num = cmd.executeUpdate();

      if (num == 0) {
        throw new ProfessorInexistenteException(professor.getCodigo());
      }
      cmd.close();
    } catch (ProfessorInexistenteException e) {
      throw e;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        poolBD.returnConnection(con);
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Exclui professor do sistema pelo Codigo.
   * @param codigo
   */
  public void excluirProfessor(int codigo) throws SIAlocacaoException,
      ProfessorInexistenteException, ProfessorAlocadoException{

    Connection con = null;
    try {
      con = poolBD.getConnection();

      PreparedStatement cmd =
          con.prepareStatement("DELETE FROM professor WHERE " +
                                           "codigo=?");
      cmd.setInt(1, codigo);

      int num = cmd.executeUpdate();

      if (num == 0) {
        throw new ProfessorInexistenteException(codigo);
      }
      cmd.close();
    }catch (ProfessorInexistenteException e) {
      throw e;
    }catch (SQLException e) {
      if(e.getMessage().indexOf("referential integrity") >= 0)
        throw new ProfessorAlocadoException();
    }
    catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        poolBD.returnConnection(con);
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Busca professor no sistema pelo Codigo.
   * @param codigo
   * @return Professor
   */
  public Professor buscarProfessor(int codigo) throws SIAlocacaoException,
      ProfessorInexistenteException {

    Connection con = null;
    try {
      con = poolBD.getConnection();
      PreparedStatement cmd =
          con.prepareStatement("SELECT nome, cpf FROM professor " +
                                           "WHERE codigo=?");

      cmd.setInt(1, codigo);

      ResultSet result = cmd.executeQuery();

      Professor professor;

      if (result.next()) {
        professor =
            new Professor(codigo,
                          result.getString("nome"),
                          result.getString("cpf"));
      } else {
        throw new ProfessorInexistenteException(codigo);
      }
      result.close();
      cmd.close();
      return professor;
    } catch (ProfessorInexistenteException e) {
      throw e;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        poolBD.returnConnection(con);
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

/*
   * Busca professor no sistema pelo nome.
   * @param nome
   * @return Professor
   */
  public Professor buscarProfessor(String nome) throws SIAlocacaoException,
      ProfessorInexistenteException {

    Connection con = null;
    try {
      con = poolBD.getConnection();
      PreparedStatement cmd =
          con.prepareStatement("SELECT codigo, cpf FROM professor " +
                                           "WHERE nome=?");

      cmd.setString(1, nome);

      ResultSet result = cmd.executeQuery();

      Professor professor;

      if (result.next()) {
        professor =
            new Professor(result.getInt("codigo"),
                          nome,
                          result.getString("cpf"));
      } else {
        throw new ProfessorInexistenteException(nome);
      }

      result.close();
      cmd.close();
      return professor;
    } catch (ProfessorInexistenteException e) {
      throw e;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        poolBD.returnConnection(con);
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
}

  /*
   * Lista professores do sistema.
   * @return ArrayList
   */
  public ArrayList listarProfessores() throws SIAlocacaoException {

    Connection con = null;
    try {
      con = poolBD.getConnection();
      PreparedStatement cmd =
          con.prepareStatement("SELECT * FROM professor");

      ResultSet result = cmd.executeQuery();

      ArrayList professores =
          new ArrayList();

      while (result.next()) {
        professores.add(
            new Professor(result.getInt("codigo"),
                          result.getString("nome"),
                          result.getString("cpf")));
      }
      result.close();
      cmd.close();
      return professores;

    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        poolBD.returnConnection(con);
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  public Map buscarPrioridades(Professor professor) throws SIAlocacaoException {
     Map prioridades = new HashMap();

    Connection con = null;
    try {
      con = poolBD.getConnection();

       PreparedStatement cmd =
         con.prepareStatement("SELECT codigoHorario, prioridade "+
           "FROM professor_horario WHERE codigoProfessor=?");

       cmd.setInt(1, professor.getCodigo());

       ResultSet result =
         cmd.executeQuery();

       while(result.next()) {
         prioridades.put(result.getString("codigoHorario"),
           result.getString("prioridade"));
       }
      result.close();
      cmd.close();
      return prioridades;
     } catch (Exception e) {
       throw new SIAlocacaoException(e.getMessage(), e);
     } finally {
      try {
        poolBD.returnConnection(con);
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
     }
   }

  /*
   * Lista as preferencias de horarios dos professores.
   * @return ArrayList com objetos Integer com preferencias por horarios
   * ordenados por codigo do horario
   */
  public ArrayList listarPreferencias(int codProfessor) throws SIAlocacaoException {

    Connection con = null;
    try {
      con = poolBD.getConnection();

      PreparedStatement cmd =
          con.prepareStatement("SELECT prioridade FROM "+
            "professor_horario WHERE codigoprofessor=? ORDER BY codigohorario");

      cmd.setInt(1, codProfessor);

      ResultSet result = cmd.executeQuery();

      ArrayList preferencias =
          new ArrayList();

      while (result.next()) {
        preferencias.add(
            new Integer(result.getInt("prioridade")) );
      }

      result.close();
      cmd.close();
      return preferencias;

    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        poolBD.returnConnection(con);
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Insere os horarios  preferenciais do professor no sistema.
   * @param codigoProfessor
   * @param codigoHorario
   * @param preferencia
   */
   public void inserirHorariosProfessor(int codigoProfessor, int codigoHorario,
     int prioridade) throws SIAlocacaoException {

    Connection con = null;
    try {
      con = poolBD.getConnection();
      PreparedStatement cmd =
          con.prepareStatement("INSERT INTO professor_horario (" +
                                           "codigoProfessor, codigoHorario," +
                                           "prioridade) VALUES (?, ?, ?)");

      cmd.setInt(1, codigoProfessor);
      cmd.setInt(2, codigoHorario);
      cmd.setInt(3, prioridade);

      cmd.executeUpdate();

      cmd.close();
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        poolBD.returnConnection(con);
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  } // inserirHorariosProfessor

/*
   * Atualiza os horarios  preferenciais do professor no sistema.
   * @param codigoProfessor
   * @param codigoHorario
   * @param preferencia
   */
   public void alterarHorariosProfessor(int codigoProfessor, int codigoHorario,
     int prioridade) throws SIAlocacaoException {

    Connection con = null;
    try {
      con = poolBD.getConnection();
      PreparedStatement cmd =
          con.prepareStatement("UPDATE professor_horario SET "+
            "prioridade=? WHERE codigoprofessor=? and codigohorario=?");

      cmd.setInt(1, prioridade);
      cmd.setInt(2, codigoProfessor);
      cmd.setInt(3, codigoHorario);

      cmd.executeUpdate();
      cmd.close();
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        poolBD.returnConnection(con);
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  } // alterarHorariosProfessor
}
