package sialocacao.sala.database;

/**
 * <p>Title: SalaDatabase.java </p>
 * <p>Description: Implementa a camada de persistncia de sala. </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III </p>
 * @author: Carlos A. Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 1.0
 */

import java.sql.*;
import java.util.*;

import sialocacao.*;
import sialocacao.sala.*;
import sialocacao.sala.excecoes.*;

public class SalaDatabase implements ISalaDatabase {

  private Database poolBD;

  /*
   * Construtor para carregar driver jdbc.
   */
  public SalaDatabase() throws Exception {
    poolBD = Database.instance();
  }

  /*
   * Insere sala no sistema.
   * @param descricao
   * @param localizacao
   * @param observacao
   */
  public void inserirSala(String descricao, String localizacao,
    String observacao, int codigoCategoria) throws SIAlocacaoException {

    Connection con = null;
    try {
      con = poolBD.getConnection();
      PreparedStatement cmd =
          con.prepareStatement("INSERT INTO sala (descricao," +
           "localizacao, observacao, codigocategoria) VALUES (?, ?, ?, ?)");

      cmd.setString(1, descricao);
      cmd.setString(2, localizacao);
      cmd.setString(3, observacao);
      cmd.setInt(4, codigoCategoria);

      cmd.executeUpdate();
      cmd.close();
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
        try {
          poolBD.returnConnection(con);
        } catch (Exception e) {
          throw new SIAlocacaoException(e.getMessage(), e);
        }
    }
  }

  /*
   * Altera sala no sistema.
   * @param sala
   */
  public void alterarSala(Sala sala) throws SIAlocacaoException,
      SalaInexistenteException {

    Connection con = null;
    try {
      con = poolBD.getConnection();
      PreparedStatement cmd =
          con.prepareStatement(
          "UPDATE sala SET descricao=?, localizacao=?, observacao=?, "+
          "codigocategoria=? WHERE codigo=?");

      cmd.setString(1, sala.getDescricao());
      cmd.setString(2, sala.getLocalizacao());
      cmd.setString(3, sala.getObservacao());
      cmd.setInt(4, sala.getCodigoCategoria());
      cmd.setInt(5, sala.getCodigo());

      int num = cmd.executeUpdate();

      if (num == 0) {
        throw new SalaInexistenteException(sala.getCodigo());
      }
      cmd.close();
    } catch (SalaInexistenteException e) {
      throw e;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
        try {
          poolBD.returnConnection(con);
        } catch (Exception e) {
          throw new SIAlocacaoException(e.getMessage(), e);
        }
    }
  }

  /*
   * Exclui sala do sistema pelo cdigo.
   * @param codigo
   */
  public void excluirSala(int codigo) throws SIAlocacaoException,
      SalaInexistenteException {

    Connection con = null;
    try {
      con = poolBD.getConnection();
      PreparedStatement cmd =
          con.prepareStatement("DELETE FROM sala WHERE " +
                                           "codigo=?");

      cmd.setInt(1, codigo);

      int num = cmd.executeUpdate();

      if (num == 0) {
        throw new SalaInexistenteException(codigo);
      }
      cmd.close();
    } catch (SQLException e) {
      if(e.getMessage().indexOf("referential integrity") >= 0)
        throw new SalaAlocadaException();
    } catch (SalaInexistenteException e) {
      throw e;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
        try {
          poolBD.returnConnection(con);
        } catch (Exception e) {
          throw new SIAlocacaoException(e.getMessage(), e);
        }
    }
  }

  /*
   * Busca sala no sistema pelo cdigo.
   * @param codigo
   * @return Sala
   */
  public Sala buscarSala(int codigo) throws SIAlocacaoException,
      SalaInexistenteException {

    Connection con = null;
    try {
      con = poolBD.getConnection();
      PreparedStatement cmd =
          con.prepareStatement("SELECT descricao, localizacao, "+
           "observacao, codigocategoria FROM sala WHERE codigo=?");

      cmd.setInt(1, codigo);

      ResultSet result = cmd.executeQuery();

      Sala Sala;

      if (result.next()) {
        Sala = new Sala(codigo,
                  result.getString("descricao"),
                  result.getString("localizacao"),
                  result.getString("observacao"),
                  result.getInt("codigocategoria"));
      } else {
        throw new SalaInexistenteException(codigo);
      }
      result.close();
      cmd.close();
      return Sala;

    } catch (SalaInexistenteException e) {
      throw e;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
        try {
          poolBD.returnConnection(con);
        } catch (Exception e) {
          throw new SIAlocacaoException(e.getMessage(), e);
        }
    }
  }

  /*
   * Busca sala no sistema pelo cdigo.
   * @param descricao
   * @return Sala
   */
  public Sala buscarSala(String descricao) throws SIAlocacaoException,
      SalaInexistenteException {

    Connection con = null;
    try {
      con = poolBD.getConnection();
      PreparedStatement cmd =
          con.prepareStatement("SELECT codigo, localizacao, "+
           "observacao, codigocategoria FROM sala WHERE descricao=?");

      cmd.setString(1, descricao);

      ResultSet result = cmd.executeQuery();

      Sala Sala;

      if (result.next()) {
        Sala = new Sala(result.getInt("codigo"),
                  descricao,
                  result.getString("localizacao"),
                  result.getString("observacao"),
                  result.getInt("codigocategoria"));
      } else {
        throw new SalaInexistenteException(descricao);
      }
      result.close();
      cmd.close();
      return Sala;

    } catch (SalaInexistenteException e) {
      throw e;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
        try {
          poolBD.returnConnection(con);
        } catch (Exception e) {
          throw new SIAlocacaoException(e.getMessage(), e);
        }
    }
  }

  /*
   * Lista salas do sistema.
   * @return ArrayList
   */
  public ArrayList listarSalas() throws SIAlocacaoException {

    Connection con = null;
    try {
      con = poolBD.getConnection();
      PreparedStatement cmd =
          con.prepareStatement("SELECT * FROM sala");

      ResultSet result = cmd.executeQuery();

      ArrayList salas =
          new ArrayList();

      while (result.next()) {
        salas.add(
                  new Sala(result.getInt("codigo"),
                     result.getString("descricao"),
                     result.getString("localizacao"),
                     result.getString("observacao"),
                     result.getInt("codigocategoria")) );
      }
      result.close();
      cmd.close();
      return salas;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
        try {
          poolBD.returnConnection(con);
        } catch (Exception e) {
          throw new SIAlocacaoException(e.getMessage(), e);
        }
    }
  }

}//SalaDatabase
