package sialocacao.timetabling;

/**
 * <p>Title: TimetablingGenetico.java </p>
 * <p>Description: Algoritmo gentico para soluo do problema de timetabling.</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III </p>
 * @author: Carlos A. Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 1.0
 */

import java.util.*;
import sialocacao.*;
import sialocacao.util.*;
import sialocacao.alocacao.*;
import sialocacao.disciplina.*;
import sialocacao.sala.*;
import sialocacao.professor.*;

public class TimetablingGenetico implements AlgoritmoTimetabling {

  static {
     System.loadLibrary("genetico");
  }


  public static final int PREFERENCIA_ALTA = 2;
  public static final int PREFERENCIA_MEDIA = 1;
  public static final int PREFERENCIA_BAIXA = 0;
  /**
   * Mtodo que prope uma sugesto de alocao, levando em considerao as prioridades dos
   * professores com relao aos horrios e a adequao da sala em que as disciplinas sero
   * ministradas.
   *
   * @param disciplinasProfessores Relao identificando as associaes disciplina-professor que
   * devem ser consideradas. Armazenar no Map: key = codigoDisciplina, value = codigoProfessor
   *
   * @param preferenciasProfessores Relao da disponibilidade do professor de acordo com os
   * horrios. Armazenar no Map: key = codigoProfessor, value = Map(codigoHorario -> prioridade)
   *
   * @param adequacaoDisciplinasSalas Relao com adequao de categorias de salas a disciplinas.
   * Armazenar no Map: key = codigoDisciplina, value = Map(codigoCategoria -> adequao)
   *
   * @param salasCategorias Relao identificando a que categoria uma determinada sala
   * pertence. Armazenar no Map: key = codigoSala, value = codigoCategoria
   *
   * @param aulasDisciplina Nmero de aulas por disciplina. Armazenar no Map:
   * key = codigoDisciplina, value = numeroDeAulas.
   *
   * @param diasUteis Quantidade de dias uteis que deve ser considerada.
   *
   * @param salas Conjunto de salas disponveis para alocao.
   *
   * @param horarios Conjunto de horrios passveis de alocao
   *
   * @param professores Conjunto de professores existentes.
   *
   * @param disciplinas Conjunto de disciplinas existentes.
   *
   * @return Uma coleo com todos os elementos da alocao proposta.
   */
  public Collection proporAlocacao(Map disciplinasProfessores, Map preferenciasProfessores,
			Map adequacaoDisciplinasSalas, Map salasCategorias, Map aulasDisciplinas,
			int diasUteis, List salas, List horarios, List professores, List disciplinas){

    // Limpa os possiveis parametros anteriores do algoritmo
    this.novaConfiguracao();

    // Imforma ao algoritmo o numero de dias uteis
    this.setDiasSemana(diasUteis);
    int numProfessores = professores.size();
    int numHorarios = horarios.size();
    int numDisciplinas = disciplinas.size();
    int numSalas = salas.size();

    // Ordena os horrios de acordo com o dia da semana e hora incio
    Collections.sort(horarios);
    int [] codHorarios = new int[numHorarios];
    Iterator itHorarios = horarios.iterator();
    for (int i = 0; i < numHorarios; i++) {
      Horario horario = (Horario)itHorarios.next();
      codHorarios[i] = horario.getCodigo();
    }
    // Repassa os codigos dos horarios para o algoritmo
    this.setHorarios(codHorarios);

    int [] codSalas = new int[numSalas];
    Iterator itSalas = salas.iterator();
    for (int i = 0; i < numSalas; i++) {
      Sala sala = (Sala)itSalas.next();
      codSalas[i] = sala.getCodigo();
    }
    // Repassa os codigos das salas para o algoritmo
    this.setSalas(codSalas);

    Iterator itProf = professores.iterator();
    for (int i = 0; i < numProfessores; i++) {
      Professor prof = (Professor)itProf.next();
      Map prefHorarios = (Map)preferenciasProfessores.get(new Integer(prof.getCodigo()));
      int [] prefProfessor = new int[numHorarios];
      Iterator iterHorarios = horarios.iterator();
      for (int j = 0; j < numHorarios; j++) {
        Horario horario = (Horario)iterHorarios.next();
        String preferencia = (String)prefHorarios.get(horario.getCodigo()+"");
        if (preferencia != null)
          prefProfessor[j] = Integer.parseInt(preferencia);
        else
          prefProfessor[j] = PREFERENCIA_MEDIA;
      }
      // Repassa pro algoritmo o professor e suas preferncias de horrios
      this.inserirProfessor(prof.getCodigo(), prefProfessor);
    }

    Iterator itDisc = disciplinas.iterator();
    for (int i = 0; i < numDisciplinas; i++) {
      Disciplina disc = (Disciplina)itDisc.next();
      Integer codProf = (Integer)disciplinasProfessores.get(new Integer(disc.getCodigo()));
      Integer numAulas = (Integer)aulasDisciplinas.get(new Integer(disc.getCodigo()));

      Map prefCategorias = (Map)adequacaoDisciplinasSalas.get(new Integer(disc.getCodigo()));
      int [] prefDisciplina = new int[numSalas];
      Iterator iterSalas = salas.iterator();
      for (int j = 0; j < numSalas; j++) {
        Sala sala = (Sala)iterSalas.next();
        Integer categoria = (Integer)salasCategorias.get(new Integer(sala.getCodigo()));
        String preferencia = (String)prefCategorias.get(categoria.toString());
        if (preferencia != null)
          prefDisciplina[j] = Integer.parseInt(preferencia);
        else
          prefDisciplina[j] = PREFERENCIA_MEDIA;
      }
      // Repassa pro algoritmo a disciplina e suas preferncias de salas
      this.inserirDisciplina(disc.getCodigo(), prefDisciplina);
    }

    itDisc = disciplinas.iterator();
    for (int i = 0; i < numDisciplinas; i++) {
      Disciplina disc = (Disciplina)itDisc.next();
      Integer codProf = (Integer)disciplinasProfessores.get(new Integer(disc.getCodigo()));
       Integer numAulas = (Integer)aulasDisciplinas.get(new Integer(disc.getCodigo()));
      // Repassa pro algoritmo as associaes disciplina-professor a serem alocadas
      this.inserirAula(codProf.intValue(), disc.getCodigo(), numAulas.intValue());
    }


       ArrayList alocacoes = new ArrayList();

    int [][] alocacao = this.alocar();
    for(int i=0; i<alocacao.length; i++)
     {
       alocacoes.add( new Alocacao ( 0,                //Codigo da Alocacao
                                     alocacao[i][1],   //Codigo da Sala
                                     alocacao[i][0],   //Codigo do Horario
                                     alocacao[i][3],   //Codigo da Disciplina
                                     alocacao[i][2],   //Codigo do Professor
                                     new Data()
                                    )
                     );

     }

    return alocacoes;
  }

  private native void novaConfiguracao();

  private native void setDiasSemana(int diasDaSemana);

  private native void setSalas(int [] codSalas);

  private native void setHorarios(int [] codHorarios);

  private native void inserirProfessor(int codProfessor, int [] preferencias);

  private native void inserirDisciplina(int codDisciplina, int [] preferencias);

  private native void inserirAula(int codProfessor, int codDisciplina, int numAulas);

  private native int[][]  alocar();


}
