package sialocacao.timetabling;

/**
 * <p>Title: TimetablingGenetico.java </p>
 * <p>Description: Algoritmo gentico para soluo do problema de timetabling.</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III </p>
 * @author: Carlos A. Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 1.0
 */

import java.util.*;
import sialocacao.*;
import sialocacao.alocacao.*;
import sialocacao.disciplina.*;
import sialocacao.sala.*;
import sialocacao.professor.*;

public class TimetablingGeneticoVelho implements AlgoritmoTimetabling {

  public static final int PREFERENCIA_ALTA = 1;
  public static final int PREFERENCIA_MEDIA = 2;
  public static final int PREFERENCIA_BAIXA = 3;
  /**
   * Mtodo que prope uma sugesto de alocao, levando em considerao as prioridades dos
   * professores com relao aos horrios e a adequao da sala em que as disciplinas sero
   * ministradas.
   *
   * @param disciplinasProfessores Relao identificando as associaes disciplina-professor que
   * devem ser consideradas. Armazenar no Map: key = codigoDisciplina, value = codigoProfessor
   *
   * @param preferenciasProfessores Relao da disponibilidade do professor de acordo com os
   * horrios. Armazenar no Map: key = codigoProfessor, value = Map(codigoHorario -> prioridade)
   *
   * @param adequacaoDisciplinasSalas Relao com adequao de categorias de salas a disciplinas.
   * Armazenar no Map: key = codigoDisciplina, value = Map(codigoCategoria -> adequao)
   *
   * @param salasCategorias Relao identificando a que categoria uma determinada sala
   * pertence. Armazenar no Map: key = codigoSala, value = codigoCategoria
   *
   * @param aulasDisciplina Nmero de aulas por disciplina. Armazenar no Map:
   * key = codigoDisciplina, value = numeroDeAulas.
   *
   * @param diasUteis Quantidade de dias uteis que deve ser considerada.
   *
   * @param salas Conjunto de salas disponveis para alocao.
   *
   * @param horarios Conjunto de horrios passveis de alocao
   *
   * @param professores Conjunto de professores existentes.
   *
   * @param disciplinas Conjunto de disciplinas existentes.
   *
   * @return Uma coleo com todos os elementos da alocao proposta.
   */
  public Collection proporAlocacao(Map disciplinasProfessores, Map preferenciasProfessores,
			Map adequacaoDisciplinasSalas, Map salasCategorias, Map aulasDisciplinas,
			int diasUteis, List salas, List horarios, List professores, List disciplinas){

    int numProfessores = professores.size();
    int numHorarios = horarios.size();
    int numDisciplinas = disciplinas.size();
    int numSalas = salas.size();

	// Ordena os horrios de acordo com o dia da semana e hora incio
    Collections.sort(horarios);
	int [] codHorarios = new int[numHorarios];
	Iterator itHorarios = horarios.iterator();
	for (int i = 0; i < numHorarios; i++) {
	  Horario horario = (Horario)itHorarios.next();
	  codHorarios[i] = horario.getCodigo();
	}

	int [] codSalas = new int[numSalas];
	Iterator itSalas = salas.iterator();
	for (int i = 0; i < numSalas; i++) {
	  Sala sala = (Sala)itSalas.next();
	  codSalas[i] = sala.getCodigo();
	}

	Iterator itProf = professores.iterator();
	for (int i = 0; i < numProfessores; i++) {
	  Professor prof = (Professor)itProf.next();
	  Map prefHorarios = (Map)preferenciasProfessores.get(new Integer(prof.getCodigo()));
	  int [] prefProfessor = new int[numHorarios];
	  Iterator iterHorarios = horarios.iterator();
	  for (int j = 0; j < numHorarios; j++) {
		Horario horario = (Horario)iterHorarios.next();
		Integer preferencia = (Integer)prefHorarios.get(new Integer(horario.getCodigo()));
		if (preferencia != null)
			prefProfessor[j] = preferencia.intValue();
		else
			prefProfessor[j] = PREFERENCIA_MEDIA;
	  }
	  // Repassa pro algoritmo o professor e suas preferncias de horrios
	  this.inserirProfessor(prof.getCodigo(), prefProfessor, numHorarios);
	}

	Iterator itDisc = disciplinas.iterator();
	for (int i = 0; i < numDisciplinas; i++) {
	  Disciplina disc = (Disciplina)itDisc.next();
	  Integer codProf = (Integer)disciplinasProfessores.get(new Integer(disc.getCodigo()));
	  Integer numAulas = (Integer)aulasDisciplinas.get(new Integer(disc.getCodigo()));

	  // Repassa pro algoritmo as associaes disciplina-professor a serem alocadas
	  this.inserirAula(disc.getCodigo(), codProf.intValue(), numAulas.intValue());

	  Map prefCategorias = (Map)adequacaoDisciplinasSalas.get(new Integer(disc.getCodigo()));
	  int [] prefDisciplina = new int[numSalas];
	  Iterator iterSalas = salas.iterator();
	  for (int j = 0; j < numSalas; j++) {
		Sala sala = (Sala)iterSalas.next();
		Integer categoria = (Integer)salasCategorias.get(new Integer(sala.getCodigo()));
		Integer preferencia = (Integer)prefCategorias.get(categoria);
		if (preferencia != null)
		  prefDisciplina[j] = preferencia.intValue();
		else
		  prefDisciplina[j] = PREFERENCIA_MEDIA;
	  }
	  // Repassa pro algoritmo a disciplina e suas preferncias de salas
	  this.inserirDisciplina(disc.getCodigo(), prefDisciplina, numSalas);
	}

	// Demais parmeteros a serem passados ao algoritmo
	this.setHorarios(codHorarios, numHorarios);
	this.setSalas(codSalas, numSalas);
	this.setDiasSemana(diasUteis);

	// CODIGO TEMPORARIO - APENAS PARA TESTES ENQUANTO TIMETABLING NO EST OK!
	SIAlocacao sialocacao = new SIAlocacao();
	ArrayList alocacoes = new ArrayList();
	try {
		alocacoes = sialocacao.listarAlocacoes();
	} catch(SIAlocacaoException e){
	}

    return alocacoes;
  }

  private void inserirProfessor(int codProfessor, int [] preferencias, int numHorarios) {
  }

  private void inserirDisciplina(int codDisciplina, int [] preferencias, int numSalas) {
  }

  private void inserirAula(int codDisciplina, int codProfessor, int numAulas) {
  }

  private void setSalas(int [] codSalas, int numSalas) {
  }

  private void setHorarios(int [] codHorarios, int numHorarios) {
  }

  private void setDiasSemana(int diasDaSemana) {
  }

}
